@@ -24,6 +24,8 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
+#include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>             /* malloc() */
 #include <string.h>             /* memcpy() */
@@ -38,8 +40,21 @@
 #include "net.h"
 #include "skin.h"
 
-#define PVER "1.0.3"
-#define PDATE "2014-2023"
+#ifndef PACKAGE_VERSION
+#define PACKAGE_VERSION __DATE__ " DEV"
+#endif
+
+#ifndef PKGDATADIR
+#define PKGDATADIR "/usr/share/simplesok"
+#endif
+
+#ifndef SDL_RENDERER_FLAGS
+#define SDL_RENDERER_FLAGS 0
+#endif
+
+#define TEXTIFY(text) #text
+#define TEXTIFY_VALUE(value) TEXTIFY(value)
+#define PDATE "2014-" TEXTIFY_VALUE(PACKAGE_YEAR)
 
 #define INET_HOST "simplesok.osdn.io"
 #define INET_PORT 80
@@ -823,7 +838,7 @@ static unsigned char *selectgametype(SDL_Renderer *renderer, struct spritesstruc
           int tity;
           const char *simpstr = "simple";
           const char *sokostr = "SOKOBAN";
-          const char *verstr = "ver " PVER;
+          const char *verstr = "ver " PACKAGE_VERSION;
           const char *copystr = "Copyright (C) " PDATE " Mateusz Viste";
 
           get_string_size(simpstr, 100, sprites, &simpw, &simph);
@@ -1379,7 +1394,7 @@ static int parse_cmdline(struct videosettings *settings, int argc, char **argv,
       } else if ((*levelfile == NULL) && (argv[i][0] != '-')) { /* else assume it is a level file */
         *levelfile = strdup(argv[i]);
       } else { /* invalid argument */
-        puts("Simple Sokoban ver " PVER);
+        puts("Simple Sokoban ver " PACKAGE_VERSION);
         puts("Copyright (C) " PDATE " Mateusz Viste");
         puts("");
         puts("usage: simplesok [options] [levelfile.xsb]");
@@ -1390,10 +1405,10 @@ static int parse_cmdline(struct videosettings *settings, int argc, char **argv,
         puts("  --skin=name         skin name to be used (default: antique3)");
         puts("  --skinlist          display the list of installed skins");
         puts("");
-        puts("Skin files can be are stored in a couple of different directories:");
-        puts(" * a skins/ subdirectory in SimpleSok's application directory");
-        puts(" * /usr/share/simplesok/skins/");
+        puts("Skin files can be located in the following directories:");
         puts(" * a skins/ subdirectory in SimpleSok's user directory");
+        puts(" * a skins/ subdirectory in SimpleSok's application directory");
+        puts(" * " PKGDATADIR "/skins/");
         puts("");
         puts("If skin loading fails, then a default (embedded) skin is used.");
         puts("");
@@ -1447,7 +1462,10 @@ int main(int argc, char **argv) {
    * "2" or "best"     - anisotropic filtering */
   SDL_SetHint(SDL_HINT_RENDER_SCALE_QUALITY, "0");
 
-  window = SDL_CreateWindow("Simple Sokoban " PVER, SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED, SCREEN_DEFAULT_WIDTH, SCREEN_DEFAULT_HEIGHT, SDL_WINDOW_SHOWN | SDL_WINDOW_RESIZABLE);
+  window = SDL_CreateWindow("Simple Sokoban " PACKAGE_VERSION,
+                            SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED,
+                            SCREEN_DEFAULT_WIDTH, SCREEN_DEFAULT_HEIGHT,
+                            SDL_WINDOW_SHOWN | SDL_WINDOW_RESIZABLE);
   if (window == NULL) {
     printf("Window could not be created! SDL_Error: %s\n", SDL_GetError());
     return(1);
@@ -1456,7 +1474,7 @@ int main(int argc, char **argv) {
   setsokicon(window);
   SDL_SetWindowMinimumSize(window, 160, 120);
 
-  renderer = SDL_CreateRenderer(window, -1, 0);
+  renderer = SDL_CreateRenderer(window, -1, SDL_RENDERER_FLAGS);
   if (renderer == NULL) {
     SDL_DestroyWindow(window);
     printf("Renderer could not be created! SDL_Error: %s\n", SDL_GetError());
