@@ -24,9 +24,12 @@
  * SOFTWARE.
  */
 
+#include "config.h"
+
 #include <dirent.h>
 #include <errno.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include <string.h>
 
 #include "data.h"
@@ -36,6 +39,10 @@
 
 #include "skin.h"
 
+#ifndef PKGDATADIR
+#define PKGDATADIR "/usr/share/simplesok"
+#endif
+
 
 /* loads a bmp.gz graphic and returns it as a texture, NULL on error */
 static SDL_Texture *loadGraphic(SDL_Renderer *renderer, const void *memptr, size_t memlen) {
@@ -97,80 +104,85 @@ void skin_list_free(struct skinlist *l) {
 }
 
 
-/* filter out skin names to match bmp.gz files only */
+/* filter out skin names to match *.bmp.gz files only */
 static int skin_filter(const struct dirent *d) {
-  char *ext = strstr(d->d_name, ".bmp.gz");
-  if ((ext == NULL) || (strlen(ext) != 7)) return(0);
+  int slen = strlen(d->d_name);
+  if (slen <= 7) return(0);
+  if (strcmp(d->d_name + slen - 7, ".bmp.gz") != 0) return(0);
   return(1);
 }
 
 
+/* Load skin names and path from directory. */
+static void skin_list_from_dir(struct skinlist **head, const char *dir) {
+  DIR *dirfd = opendir(dir);
+
+  if (dirfd != NULL) {
+    struct dirent *dentry;
+
+    while ((dentry = readdir(dirfd)) != NULL) {
+      char fname[512];
+
+      snprintf(fname, sizeof(fname), "%s%s", dir, dentry->d_name);
+      if (skin_filter(dentry) && !access(fname, R_OK)) {
+        char *name = strdup(dentry->d_name);
+        int i = 1;
+        size_t len;
+        struct skinlist *node;
+        struct skinlist **p;
+
+        /* trim file extension from skin name */
+        len = strlen(name);
+        if (len > 7) name[len - 7] = '\0';
+
+        /* Prepare to insert in alphanumerical order. */
+        for (p = head; (node = *p) && (i = strcmp(name, node->name)) > 0;) {
+          p = &node->next;
+        }
+
+        if (i != 0) {  /* Ignore duplicates. */
+          /* prep new node */
+          node = malloc(sizeof(struct skinlist));
+          node->name = name;
+          node->path = strdup(fname);
+
+          /* insert new node in list of results */
+          node->next = *p;
+          *p = node;
+        } else {
+          free(name);
+        }
+      }
+    }
+    closedir(dirfd);
+  }
+}
+
+
 struct skinlist *skin_list(void) {
-  char *dirs[4];
-  int i;
-  struct skinlist *r = NULL;
+  struct skinlist *head = NULL;
   char *ptr;
   char buff[512];
 
-  /* directories where I should look for skins */
-  i = 0;
-  /* applications running path */
-  ptr = SDL_GetBasePath();
-  if (ptr != NULL) {
-    snprintf(buff, sizeof(buff), "%sskins/", ptr);
-    dirs[i++] = strdup(buff);
-    SDL_free(ptr);
-  }
-  /* /usr/share/... */
-  dirs[i++] = strdup("/usr/share/simplesok/skins/");
   /* local user preferences directory */
   ptr = SDL_GetPrefPath("", "simplesok");
   if (ptr != NULL) {
     snprintf(buff, sizeof(buff), "%sskins/", ptr);
-    dirs[i++] = strdup(buff);
+    skin_list_from_dir(&head, buff);
     SDL_free(ptr);
   }
-  /* append an end-of-list marker */
-  dirs[i] = NULL;
 
-  /* look in each directory (and free the dir string) */
-  for (i = 0; dirs[i] != NULL; i++) {
-    DIR *dirfd;
-    struct dirent *dentry;
-
-    dirfd = opendir(dirs[i]);
-    if (dirfd == NULL) continue;
-
-    while ((dentry = readdir(dirfd)) != NULL) {
-      char fname[512];
-      struct skinlist *node;
-
-      if (skin_filter(dentry) == 0) continue;
-
-      /* prep new node */
-      snprintf(fname, sizeof(fname), "%s%s", dirs[i], dentry->d_name);
-      node = calloc(1, sizeof(struct skinlist));
-      node->name = strdup(dentry->d_name);
-      node->path = strdup(fname);
-
-      /* trim file extension from skin name */
-      {
-        size_t len = strlen(node->name);
-        if (len > 7) node->name[len - 7] = 0;
-      }
-
-      /* append new node to list of results */
-      node->next = r;
-      r = node;
-
-    }
-    closedir(dirfd);
+  /* applications running path */
+  ptr = SDL_GetBasePath();
+  if (ptr != NULL) {
+    snprintf(buff, sizeof(buff), "%sskins/", ptr);
+    skin_list_from_dir(&head, buff);
+    SDL_free(ptr);
   }
 
-  /* free dirs entries */
-  for (i = 0; dirs[i] != NULL; i++) free(dirs[i]);
-
-  return(r);
+  skin_list_from_dir(&head, PKGDATADIR "/skins/");
+  skin_list_from_dir(&head, "/usr/share/simplesok/skins/");
+  return(head);
 }
 
 
